@@ -405,9 +405,31 @@ class EncryptionAlgorithmId(ObjectIdentifier):
         '1.2.840.113549.3.7': 'tripledes_3key',
         '1.2.840.113549.3.2': 'rc2',
         '1.2.840.113549.3.9': 'rc5',
-        '2.16.840.1.101.3.4.1.2': 'aes128',
-        '2.16.840.1.101.3.4.1.22': 'aes192',
-        '2.16.840.1.101.3.4.1.42': 'aes256',
+        # From http://csrc.nist.gov/groups/ST/crypto_apps_infra/csor/algorithms.html#AES
+        '2.16.840.1.101.3.4.1.1': 'aes128_ecb',
+        '2.16.840.1.101.3.4.1.2': 'aes128_cbc',
+        '2.16.840.1.101.3.4.1.3': 'aes128_ofb',
+        '2.16.840.1.101.3.4.1.4': 'aes128_cfb',
+        '2.16.840.1.101.3.4.1.5': 'aes128_wrap',
+        '2.16.840.1.101.3.4.1.6': 'aes128_gcm',
+        '2.16.840.1.101.3.4.1.7': 'aes128_ccm',
+        '2.16.840.1.101.3.4.1.8': 'aes128_wrap_pad',
+        '2.16.840.1.101.3.4.1.21': 'aes192_ecb',
+        '2.16.840.1.101.3.4.1.22': 'aes192_cbc',
+        '2.16.840.1.101.3.4.1.23': 'aes192_ofb',
+        '2.16.840.1.101.3.4.1.24': 'aes192_cfb',
+        '2.16.840.1.101.3.4.1.25': 'aes192_wrap',
+        '2.16.840.1.101.3.4.1.26': 'aes192_gcm',
+        '2.16.840.1.101.3.4.1.27': 'aes192_ccm',
+        '2.16.840.1.101.3.4.1.28': 'aes192_wrap_pad',
+        '2.16.840.1.101.3.4.1.41': 'aes256_ecb',
+        '2.16.840.1.101.3.4.1.42': 'aes256_cbc',
+        '2.16.840.1.101.3.4.1.43': 'aes256_ofb',
+        '2.16.840.1.101.3.4.1.44': 'aes256_cfb',
+        '2.16.840.1.101.3.4.1.45': 'aes256_wrap',
+        '2.16.840.1.101.3.4.1.46': 'aes256_gcm',
+        '2.16.840.1.101.3.4.1.47': 'aes256_ccm',
+        '2.16.840.1.101.3.4.1.48': 'aes256_wrap_pad',
         # From PKCS#5
         '1.2.840.113549.1.5.13': 'pbes2',
         '1.2.840.113549.1.5.1': 'pbes1_md2_des',
@@ -441,9 +463,12 @@ class EncryptionAlgorithm(Sequence):
         'tripledes_3key': OctetString,
         'rc2': Rc2Params,
         'rc5': Rc5Params,
-        'aes128': OctetString,
-        'aes192': OctetString,
-        'aes256': OctetString,
+        'aes128_cbc': OctetString,
+        'aes192_cbc': OctetString,
+        'aes256_cbc': OctetString,
+        'aes128_ofb': OctetString,
+        'aes192_ofb': OctetString,
+        'aes256_ofb': OctetString,
         # From PKCS#5
         'pbes1_md2_des': Pbes1Params,
         'pbes1_md5_des': Pbes1Params,
@@ -636,12 +661,16 @@ class EncryptionAlgorithm(Sequence):
 
         encryption_algo = self['algorithm'].native
 
+        if encryption_algo[0:3] == 'aes':
+            return {
+                'aes128_': 16,
+                'aes192_': 24,
+                'aes256_': 32,
+            }[encryption_algo[0:7]]
+
         cipher_lengths = {
             'des': 8,
             'tripledes_3key': 24,
-            'aes128': 16,
-            'aes192': 24,
-            'aes256': 32,
         }
 
         if encryption_algo in cipher_lengths:
@@ -710,6 +739,40 @@ class EncryptionAlgorithm(Sequence):
             encryption_algo
         ))
 
+    @property
+    def encryption_mode(self):
+        """
+        Returns the name of the encryption mode to use.
+
+        :return:
+            A unicode string from one of the following: "cbc", "ecb", "ofb",
+            "cfb", "wrap", "gcm", "ccm", "wrap_pad"
+        """
+
+        encryption_algo = self['algorithm'].native
+
+        if encryption_algo[0:7] in set(['aes128_', 'aes192_', 'aes256_']):
+            return encryption_algo[7:]
+
+        if encryption_algo[0:6] == 'pbes1_':
+            return 'cbc'
+
+        if encryption_algo[0:7] == 'pkcs12_':
+            return 'cbc'
+
+        if encryption_algo in set(['des', 'tripledes_3key', 'rc2', 'rc5']):
+            return 'cbc'
+
+        if encryption_algo == 'pbes2':
+            return self['parameters']['encryption_scheme'].encryption_mode
+
+        raise ValueError(unwrap(
+            '''
+            Unrecognized encryption algorithm "%s"
+            ''',
+            encryption_algo
+        ))
+
     @property
     def encryption_cipher(self):
         """
@@ -724,17 +787,14 @@ class EncryptionAlgorithm(Sequence):
 
         encryption_algo = self['algorithm'].native
 
-        cipher_map = {
-            'des': 'des',
-            'tripledes_3key': 'tripledes',
-            'aes128': 'aes',
-            'aes192': 'aes',
-            'aes256': 'aes',
-            'rc2': 'rc2',
-            'rc5': 'rc5',
-        }
-        if encryption_algo in cipher_map:
-            return cipher_map[encryption_algo]
+        if encryption_algo[0:7] in set(['aes128_', 'aes192_', 'aes256_']):
+            return 'aes'
+
+        if encryption_algo in set(['des', 'rc2', 'rc5']):
+            return encryption_algo
+
+        if encryption_algo == 'tripledes_3key':
+            return 'tripledes'
 
         if encryption_algo == 'pbes2':
             return self['parameters']['encryption_scheme'].encryption_cipher
@@ -773,12 +833,12 @@ class EncryptionAlgorithm(Sequence):
 
         encryption_algo = self['algorithm'].native
 
+        if encryption_algo[0:7] in set(['aes128_', 'aes192_', 'aes256_']):
+            return 16
+
         cipher_map = {
             'des': 8,
             'tripledes_3key': 8,
-            'aes128': 16,
-            'aes192': 16,
-            'aes256': 16,
             'rc2': 8,
         }
         if encryption_algo in cipher_map:
@@ -826,11 +886,21 @@ class EncryptionAlgorithm(Sequence):
 
         encryption_algo = self['algorithm'].native
 
-        if encryption_algo in ('rc2', 'rc5'):
+        if encryption_algo in set(['rc2', 'rc5']):
             return self['parameters'].parsed['iv'].native
 
         # For DES/Triple DES and AES the IV is the entirety of the parameters
-        if encryption_algo in ('des', 'tripledes_3key', 'aes128', 'aes192', 'aes256'):
+        octet_string_iv_oids = set([
+            'des',
+            'tripledes_3key',
+            'aes128_cbc',
+            'aes192_cbc',
+            'aes256_cbc',
+            'aes128_ofb',
+            'aes192_ofb',
+            'aes256_ofb',
+        ])
+        if encryption_algo in octet_string_iv_oids:
             return self['parameters'].native
 
         if encryption_algo == 'pbes2':
